﻿/*
 * Declaration for "GuidelineItem" pseudo-namespace for Guideline item display processing
 */
var GuidelineItem = function() {};

// Guideline item descriptor content object
GuidelineItem.DataObject = function(type, subtype, text)
{
  // Parameters (do not change after generation)
  this._dataType = type;
  this._dataSubtype = subtype;
  this._dataText = text;
  // Flag indicating whether a continued line has been added (not needed?)
  this._dataContinuous = false;
}

GuidelineItem.DataObject.prototype = {
  // Get Parameters
  getType     : function () {return this._dataType;},
  getSubtype  : function () {return this._dataSubtype;},
  getText     : function () {return this._dataText;},
  // Continued line-related
  enableContinuous  : function () {this._dataContinuous = true;},
  isContinuous      : function () {return this._dataContinuous;},
  // Prepend continuing lines of text with BR tags and add them to the end of previous text
  addContinuousText : function (text) {this._dataText = this._dataText + '<br />' + text;}
};

// Guideline item object
GuidelineItem.ItemObject = function()
{
  // Data being edited
  this.SECTION_NONE = -1;
  this.currentSection = this.SECTION_NONE;
  this.objectSection = null;

  // Headings
  this.itemTitle = '';
  this.setTitle = function (title)
  {
    this.itemTitle = ((title == null) ? '' : title);
  }
  this.getTitle = function ()
  {
    return this.itemTitle;
  }

  // Guideline Item
  this.SECTION_DEFINITION = 0;
  this.arrayDefinition = new Array(0); this.arrayDefinition.length = 0;
  this.getDefinitionArray = function ()
  {
    return this.arrayDefinition;
  }
  this.getDefinitionCount = function ()
  {
    return this.arrayDefinition.length;
  }
  this.getDefinitionData = function (no)
  {
    if ((no < 0) || (no >= this.getDefinitionCount())) return null;
    return this.arrayDefinition[no];
  }
  this.addDefinitionData = function (data)
  {
    if (data != null) this.arrayDefinition.push(data);
  }

  // Test Method
  this.SECTION_TESTMETHOD = 1;
  this.arrayTestMethod = new Array(0); this.arrayTestMethod.length = 0;
  this.getTestMethodArray = function ()
  {
    return this.arrayTestMethod;
  }
  this.getTestMethodCount = function ()
  {
    return this.arrayTestMethod.length;
  }
  this.getTestMethodData = function (no)
  {
    if ((no < 0) || (no >= this.getTestMethodCount())) return null;
    return this.arrayTestMethod[no];
  }
  this.addTestMethodData = function (data)
  {
    if (data != null) this.arrayTestMethod.push(data);
  }

  // Pass/Fail Determination
  this.SECTION_CRITERION = 2;
  this.arrayCriterion  = new Array(0); this.arrayCriterion.length  = 0;
  this.getCriterionArray = function ()
  {
    return this.arrayCriterion;
  }
  this.getCriterionCount = function ()
  {
    return this.arrayCriterion.length;
  }
  this.getCriterionData = function (no)
  {
    if ((no < 0) || (no >= this.getCriterionCount())) return null;
    return this.arrayCriterion[no];
  }
  this.addCriterionData = function (data)
  {
    if (data != null) this.arrayCriterion.push(data);
  }

  // Software to Be Tested
  this.SECTION_TESTOBJECT = 3;
  this.arrayTestObject = new Array(0); this.arrayTestObject.length = 0;
  this.getTestObjectArray = function ()
  {
    return this.arrayTestObject;
  }
  this.getTestObjectCount = function ()
  {
    return this.arrayTestObject.length;
  }
  this.getTestObjectData = function (no)
  {
    if ((no < 0) || (no >= this.getTestObjectCount())) return null;
    return this.arrayTestObject[no];
  }
  this.addTestObjectData = function (data)
  {
    if (data != null) this.arrayTestObject.push(data);
  }

  // Exceptions
  this.SECTION_EXCLUSION = 4;
  this.arrayExclusion  = new Array(0); this.arrayExclusion.length  = 0;
  this.getExclusionArray = function ()
  {
    return this.arrayExclusion;
  }
  this.getExclusionCount = function ()
  {
    return this.arrayExclusion.length;
  }
  this.getExclusionData = function (no)
  {
    if ((no < 0) || (no >= this.getExclusionCount())) return null;
    return this.arrayExclusion[no];
  }
  this.addExclusionData = function (data)
  {
    if (data != null) this.arrayExclusion.push(data);
  }

  // Comments (Reserved)
  this.SECTION_REMARKS = 5;
  this.arrayRemarks  = new Array(0); this.arrayRemarks.length  = 0;
  this.getRemarksArray = function ()
  {
    return this.arrayRemarks;
  }
  this.getRemarksCount = function ()
  {
    return this.arrayRemarks.length;
  }
  this.getRemarksData = function (no)
  {
    if ((no < 0) || (no >= this.getRemarksCount())) return null;
    return this.arrayRemarks[no];
  }
  this.addRemarksData = function (data)
  {
    if (data != null) this.arrayRemarks.push(data);
  }

  // Margin (Reserved)
  this.SECTION_MARGINAL = 6;
  this.arrayMarginal  = new Array(0); this.arrayMarginal.length  = 0;
  this.getMarginalArray = function ()
  {
    return this.arrayMarginal;
  }
  this.getMarginalCount = function ()
  {
    return this.arrayMarginal.length;
  }
  this.getMarginalData = function (no)
  {
    if ((no < 0) || (no >= this.getMarginalCount())) return null;
    return this.arrayMarginal[no];
  }
  this.addMarginalData = function (data)
  {
    if (data != null) this.arrayMarginal.push(data);
  }

  // Add data being edited
  this.addObjectSection = function ()
  {
    switch (this.currentSection)
    {
    case this.SECTION_DEFINITION:
      this.addDefinitionData(this.objectSection);
      break;
    case this.SECTION_TESTMETHOD:
      this.addTestMethodData(this.objectSection);
      break;
    case this.SECTION_CRITERION:
      this.addCriterionData(this.objectSection);
      break;
    case this.SECTION_TESTOBJECT:
      this.addTestObjectData(this.objectSection);
      break;
    case this.SECTION_EXCLUSION:
      this.addExclusionData(this.objectSection);
      break;
    case this.SECTION_REMARKS:
      this.addRemarksData(this.objectSection);
      break;
    case this.SECTION_MARGINAL:
      this.addMarginalData(this.objectSection);
      break;
    default:
      alert('Error at addObjectSection()');
      break;
    }
    this.currentSection = this.SECTION_NONE;
    this.objectSection = null;
  }

  // Guideline Item
  this.startDefinitionSection = function ()
  {
    this.currentSection = this.SECTION_DEFINITION;
    this.objectSection = new Array(0);
    this.objectSection.length = 0;
  }
  this.endDefinitionSection = function ()
  {
    if (this.currentSection != this.SECTION_DEFINITION) alert('Error at endDefinitionSection()');
    this.addObjectSection();
  }

  // Test Method
  this.startTestMethodSection = function ()
  {
    this.currentSection = this.SECTION_TESTMETHOD;
    this.objectSection = new Array(0);
    this.objectSection.length = 0;
  }
  this.endTestMethodSection = function ()
  {
    if (this.currentSection != this.SECTION_TESTMETHOD) alert('Error at endTestMethodSection()');
    this.addObjectSection();
  }

  // Pass/Fail Determination
  this.startCriterionSection = function ()
  {
    this.currentSection = this.SECTION_CRITERION;
    this.objectSection = new Array(0);
    this.objectSection.length = 0;
  }
  this.endCriterionSection = function ()
  {
    if (this.currentSection != this.SECTION_CRITERION) alert('Error at endCriterionSection()');
    this.addObjectSection();
  }

  // Software to Be Tested
  this.startTestObjectSection = function ()
  {
    this.currentSection = this.SECTION_TESTOBJECT;
    this.objectSection = new Array(0);
    this.objectSection.length = 0;
  }
  this.endTestObjectSection = function ()
  {
    if (this.currentSection != this.SECTION_TESTOBJECT) alert('Error at endTestObjectSection()');
    this.addObjectSection();
  }

  // Exceptions
  this.startExclusionSection = function ()
  {
    this.currentSection = this.SECTION_EXCLUSION;
    this.objectSection = new Array(0);
    this.objectSection.length = 0;
  }
  this.endExclusionSection = function ()
  {
    if (this.currentSection != this.SECTION_EXCLUSION) alert('Error at endExclusionSection()');
    this.addObjectSection();
  }

  // Comments (Reserved)
  this.startRemarksSection = function ()
  {
    this.currentSection = this.SECTION_REMARKS;
    this.objectSection = new Array(0);
    this.objectSection.length = 0;
  }
  this.endRemarksSection = function ()
  {
    if (this.currentSection != this.SECTION_REMARKS) alert('Error at endRemarksSection()');
    this.addObjectSection();
  }

  // Margin (Reserved)
  this.startMarginalSection = function ()
  {
    this.currentSection = this.SECTION_MARGINAL;
    this.objectSection = new Array(0);
    this.objectSection.length = 0;
  }
  this.endMarginalSection = function ()
  {
    if (this.currentSection != this.SECTION_MARGINAL) alert('Error at endMarginalSection()');
    this.addObjectSection();
  }

  // Convert tag characters and similar characters to entities
  this.replaceTagCharacter = function (str)
  {
    str = str.replace(/&/g, '&amp;');
    str = str.replace(/</g, '&lt;');
    str = str.replace(/>/g, '&gt;');
    return str;
  }

  // Add text for display
  this.addItemText = function (text)
  {
    if (text == null) return;
    var strs = text.split('\n');
    for (var line = 0; line < strs.length; line++)
    {
      var str = strs[line];
      var tag = '';
      var ind = str.search(/^[ \t]*@[^\s]+\s/);
      if (ind >= 0)
      {
        ind = str.search(/@[^\s]+\s/);
        str = str.slice(ind);
        var tags = str.match(/@[^\s]+\s/);
        var len = tags[0].length;
        str = str.slice(len);
        tag = tags[0].substr(1, len - 2);
      }
      // Convert tag characters and similar characters to entities
      str = this.replaceTagCharacter(str);
      if (tag.match(/^list-/i))
      {
        var subtype = tag.slice(5);
        subtype = this.replaceTagCharacter(subtype);
        var data = new GuidelineItem.DataObject('LIST', subtype, str);
        this.objectSection.push(data);
      }
      else if (tag.match(/^sublist-/i))
      {
        var subtype = tag.slice(8);
        subtype = this.replaceTagCharacter(subtype);
        var data = new GuidelineItem.DataObject('SUBLIST', subtype, str);
        this.objectSection.push(data);
      }
      else if (tag.match(/^item-/i))
      {
        var subtype = tag.slice(5);
        subtype = this.replaceTagCharacter(subtype);
        var data = new GuidelineItem.DataObject('ITEM', subtype, str);
        this.objectSection.push(data);
      }
      else if (tag.match(/^subitem-/i))
      {
        var subtype = tag.slice(8);
        subtype = this.replaceTagCharacter(subtype);
        var data = new GuidelineItem.DataObject('SUBITEM', subtype, str);
        this.objectSection.push(data);
      }
      else if (tag.match(/^disc$/i))
      {
        var data = new GuidelineItem.DataObject('UL', 'disc', str);
        this.objectSection.push(data);
      }
      else if (tag.match(/^square$/i))
      {
        var data = new GuidelineItem.DataObject('UL', 'square', str);
        this.objectSection.push(data);
      }
      else if (tag.match(/^circle$/i))
      {
        var data = new GuidelineItem.DataObject('UL', 'circle', str);
        this.objectSection.push(data);
      }
      else if (tag.match(/^reference$/i))
      {
        var data = new GuidelineItem.DataObject('ITEM', '※', str);
        this.objectSection.push(data);
      }
      else if (tag.match(/^footnote$/i))
      {
        var data = new GuidelineItem.DataObject('SUBITEM', '-', str);
        this.objectSection.push(data);
      }
      else if (tag.match(/^indent$/i))
      {
        var data = new GuidelineItem.DataObject('INDENT', '', str);
        this.objectSection.push(data);
      }
      else if (tag.match(/^continue$/i))
      {
        if (this.objectSection.length > 0)
        {
          var data = this.objectSection[(this.objectSection.length - 1)];
          data.enableContinuous();
          data.addContinuousText(str);
        }
        else
        {
          var data = new GuidelineItem.DataObject('TEXT', '', str);
          this.objectSection.push(data);
        }
      }
      else if (tag.match(/^text$/i))
      {
        var data = new GuidelineItem.DataObject('TEXT', '', str);
        this.objectSection.push(data);
      }
      else if (tag.match(/^blank$/i))
      {
        var data = new GuidelineItem.DataObject('TEXT', '', '&nbsp;');
        this.objectSection.push(data);
      }
      else if (tag.match(/^@/i))
      {
        var data = new GuidelineItem.DataObject('TEXT', '', tag + str);
        this.objectSection.push(data);
      }
      else
      {
        var data = new GuidelineItem.DataObject('TEXT', '', str);
        this.objectSection.push(data);
      }
//      alert('TAG=[' + tag + '] TEXT=[' + str + ']');
    }
  }

  // Flag indicating whether previous item was a list/sublist/item/subitem
  this.TABLE_NONE = 0;
  this.TABLE_LIST = 1;
  this.TABLE_SUBLIST = 2;
  this.TABLE_ITEM = 3;
  this.TABLE_SUBITEM = 4;

  // Table flow control
  this.controlTable = function (prev, next)
  {
    var str = '';

    if (prev != next)
    {
      if (prev != this.TABLE_NONE) str += '</table>';
      switch (next)
      {
      case this.TABLE_LIST:
      case this.TABLE_ITEM:
        str += '<table class="itemintable">';
        break;
      case this.TABLE_SUBLIST:
      case this.TABLE_SUBITEM:
        str += '<table class="itemindent">';
        break;
      }
    }
    return str;
  }

  // Generate text for display
  this.makeItemText = function (arrayData)
  {
    var str = '';
    var tableType = this.TABLE_NONE;
    if (arrayData == null) return str;
    for (var dataNo = 0; dataNo < arrayData.length; dataNo++)
    {
      var data = arrayData[dataNo];
      if (data == null) continue;
      var dataType = data.getType();
      var dataSubtype = data.getSubtype();
      var dataText = data.getText();
      if (dataType.match(/^text$/i))
      {
        str += this.controlTable(tableType, this.TABLE_NONE);
        tableType = this.TABLE_NONE;
        str += ('<div class="itemtext">' + dataText + '</div>');
      }
      else if (dataType.match(/^list$/i))
      {
        str += this.controlTable(tableType, this.TABLE_LIST);
        tableType = this.TABLE_LIST;
        str += ('<tr><th>(' + dataSubtype + ')</th><td>' + dataText + '</td></tr>');
      }
      else if (dataType.match(/^sublist$/i))
      {
        str += this.controlTable(tableType, this.TABLE_SUBLIST);
        tableType = this.TABLE_SUBLIST;
        str += ('<tr><th>(' + dataSubtype + ')</th><td>' + dataText + '</td></tr>');
      }
      else if (dataType.match(/^ul$/i))
      {
        str += this.controlTable(tableType, this.TABLE_NONE);
        tableType = this.TABLE_NONE;
        str += ('<ul type="' + dataSubtype + '" class="itemlist"><li>' + dataText + '</li></ul>');
      }
      else if (dataType.match(/^item$/i))
      {
        str += this.controlTable(tableType, this.TABLE_ITEM);
        tableType = this.TABLE_ITEM;
        str += ('<tr><th>' + dataSubtype + '</th><td>' + dataText + '</td></tr>');
      }
      else if (dataType.match(/^subitem$/i))
      {
        str += this.controlTable(tableType, this.TABLE_SUBITEM);
        tableType = this.TABLE_SUBITEM;
        str += ('<tr><th>' + dataSubtype + '</th><td>' + dataText + '</td></tr>');
      }
      else if (dataType.match(/^indent$/i))
      {
        str += this.controlTable(tableType, this.TABLE_NONE);
        tableType = this.TABLE_NONE;
        str += ('<div class="itemindent">' + dataText + '</div>');
      }
    }
    // Last thing might be a table
    str += this.controlTable(tableType, this.TABLE_NONE);

    return str;
  }

  // Guideline Item
  this.createItemTable = function (itemNumber)
  {
    createItemTable(itemNumber, '');
  }

  // Guideline Item
  this.createItemTable = function (itemNumber, targetId)
  {
    var str = '';
    //
    str += ('<a name="' + TemplateSettings.guidelineItemAnchorPrefix + itemNumber + '"></a>');
    // ID for opening/closing
    var expandId = TemplateSettings.guidelineItemIdPrefix + itemNumber;
    // DIV for opening/closing
    str += ('<div class="openitem" id="' + expandId + '">');
    // Display title
    if (this.getTitle() != '') str += ('<h3 data-reassemble="noLabel" id="' + TemplateSettings.guidelineItemLabelPrefix + itemNumber + '">' + this.getTitle() + '</h3>');
    str += ('<div class="text">');
    str += ('<p>');
    str += ('<table class="guideline">');
    // Guideline Item
    for (var n = 0; n < this.getDefinitionCount(); n++)
    {
      str += ('<tr>');
      if (n < 1) str += ('<th class="header" rowspan="' + this.getDefinitionCount() + '">' + LocaleString.lcStrLabelDefinition + '</th>');
      str += ('<th>' + this.makeItemText(this.getDefinitionData(n)) + '</th>');
      str += ('</tr>');
    }
    // Test Method
    for (var n = 0; n < this.getTestMethodCount(); n++)
    {
      str += ('<tr>');
      if (n < 1) str += ('<td class="header" rowspan="' + this.getTestMethodCount() + '">' + LocaleString.lcStrLabelTestMethod + '</td>');
      str += ('<td>' + this.makeItemText(this.getTestMethodData(n)) + '</td>');
      str += ('</tr>');
    }
    // Pass/Fail Determination
    for (var n = 0; n < this.getCriterionCount(); n++)
    {
      str += ('<tr>');
      if (n < 1) str += ('<td class="header" rowspan="' + this.getCriterionCount() + '">' + LocaleString.lcStrLabelCriterion + '</td>');
      str += ('<td>' + this.makeItemText(this.getCriterionData(n)) + '</td>');
      str += ('</tr>');
    }
    // Software to Be Tested
    for (var n = 0; n < this.getTestObjectCount(); n++)
    {
      str += ('<tr>');
      if (n < 1) str += ('<td class="header" rowspan="' + this.getTestObjectCount() + '">' + LocaleString.lcStrLabelTestObject + '</td>');
      str += ('<td>' + this.makeItemText(this.getTestObjectData(n)) + '</td>');
      str += ('</tr>');
    }
    // Exceptions
    for (var n = 0; n < this.getExclusionCount(); n++)
    {
      str += ('<tr>');
      if (n < 1) str += ('<td class="header" rowspan="' + this.getExclusionCount() + '">' + LocaleString.lcStrLabelExclusion + '</td>');
      str += ('<td>' + this.makeItemText(this.getExclusionData(n)) + '</td>');
      str += ('</tr>');
    }
    // Comments (Reserved)
    for (var n = 0; n < this.getRemarksCount(); n++)
    {
      str += ('<tr>');
      if (n < 1) str += ('<td class="header" rowspan="' + this.getRemarksCount() + '">' + LocaleString.lcStrLabelRemarks + '</td>');
      str += ('<td>' + this.makeItemText(this.getRemarksData(n)) + '</td>');
      str += ('</tr>');
    }
    str += ('</table>');
    str += ('</p>');
    // Margin notes
    for (var n = 0; n < this.getMarginalCount(); n++)
    {
      str += ('<div class="marginaltext">');
      str += (this.makeItemText(this.getMarginalData(n)));
      str += ('</div>');
    }
    str += ('</div>');
    // DIV for opening/closing
    str += ('</div>');
    var objTarget = document.getElementById(targetId);
    // alert(targetId + '->' + objTarget);
    if ((targetId == null) || (targetId == '') || (objTarget == null))
    {
      document.writeln(str);
    }
    else
    {
      objTarget.innerHTML = str;
    }
  }

}
